load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"

;purpose: regrid hdrt hdgf hdrf and obs satellite images onto one single grid domain

gridfile1=getenv("gridfile1")
gridfile2=getenv("gridfile2")
gridfile3=getenv("gridfile3")
gridfile4=getenv("gridfile4")
case_info=getenv("case_info")


 satellite_hdrt= addfile(gridfile1,"r")
 satellite_hdrf= addfile(gridfile2,"r")
 satellite_hdgf= addfile(gridfile3,"r")
 satellite_obs= addfile(gridfile4,"r")


;process hdrt channel 4

  sat_hdrt=satellite_hdrt->IMGR_tb_b04
  lat2d = satellite_hdrt->lat
  lon2d = satellite_hdrt->lon
  sat_hdrt@lon2d=lon2d  
  sat_hdrt@lat2d=lat2d
  sat_hdrt=where(isnan_ieee(sat_hdrt) .eq. True, -999., sat_hdrt)

  hdrt_min_lat=min(lat2d)
  hdrt_max_lat=max(lat2d)
  hdrt_min_lon=min(lon2d)
  hdrt_max_lon=max(lon2d)



  hdrt_min_lat=min(sat_hdrt@lat2d)
  hdrt_max_lat=max(sat_hdrt@lat2d)
  hdrt_min_lon=min(sat_hdrt@lon2d)
  hdrt_max_lon=max(sat_hdrt@lon2d)

;process hdrf channel 4

  hdrf_vnames=getfilevarnames(satellite_hdrf)
  if (hdrf_vnames(1).eq."SBT122_P0_L8_GLL0") then 
    sat_hdrf=satellite_hdrf->SBT124_P0_L8_GLL0
  else
    sat_hdrf=satellite_hdrf->SBT114_P0_L8_GLL0
  end if

  lat_0_hdrf=satellite_hdrf->lat_0
  lon_0_hdrf=satellite_hdrf->lon_0
  sat_hdrf@lon2d=conform_dims((/901,1101/),lon_0_hdrf,1)
  sat_hdrf@lat2d=conform_dims((/901,1101/),lat_0_hdrf,0)

  hdrf_min_lat=min(sat_hdrf@lat2d)
  hdrf_max_lat=max(sat_hdrf@lat2d)
  hdrf_min_lon=min(sat_hdrf@lon2d)
  hdrf_max_lon=max(sat_hdrf@lon2d)

;process hdgf channel 2

  if (hdrf_vnames(1).eq."SBT122_P0_L8_GLL0") then
    sat_hdgf=satellite_hdgf->SBT124_P0_L8_GLL0
  else
    sat_hdgf=satellite_hdgf->SBT114_P0_L8_GLL0
  end if
  lat_0_hdgf=satellite_hdgf->lat_0
  lon_0_hdgf=satellite_hdgf->lon_0
  sat_hdgf@lon2d=conform_dims((/901,1101/),lon_0_hdgf,1)
  sat_hdgf@lat2d=conform_dims((/901,1101/),lat_0_hdgf,0)


  hdgf_min_lat=min(sat_hdgf@lat2d)
  hdgf_max_lat=max(sat_hdgf@lat2d)
  hdgf_min_lon=min(sat_hdgf@lon2d)
  hdgf_max_lon=max(sat_hdgf@lon2d)

;process obs

  sat_obs=satellite_obs->calibratedData
  lat_0=satellite_obs->lat
  lon_0=satellite_obs->lon
  sat_obs@lon2d=conform_dims((/275,800/),lon_0,1)
  sat_obs@lat2d=conform_dims((/275,800/),lat_0,0)
  sat_obs=where(isnan_ieee(sat_obs) .eq. True, 99999., sat_obs)

  obs_min_lat=min(sat_obs@lat2d)
  obs_max_lat=max(sat_obs@lat2d)
  obs_min_lon=min(sat_obs@lon2d)
  obs_max_lon=max(sat_obs@lon2d)

; what are the boundary
  minlat=max((/hdrt_min_lat,hdrf_min_lat,hdgf_min_lat,obs_min_lat/))	
  matlat=min((/hdrt_max_lat,hdrf_max_lat,hdgf_max_lat,obs_max_lat/))
  minlon=max((/hdrt_min_lon,hdrf_min_lon-360,hdgf_min_lon-360,obs_min_lon/))
  maxlon=min((/hdrt_max_lon,hdrf_max_lon-360,hdgf_max_lon-360,obs_max_lon/))

  merge_lat=fspan(minlat+0,matlat-0,300)
  merge_lon=fspan(minlon+0,maxlon-0,300)

; project all hdrt hdrf and obs to same grid
  skip=1
  regrid_obs=rcm2rgrid(sat_obs@lat2d,sat_obs@lon2d,sat_obs(0,:,:),merge_lat,merge_lon,1)
  regrid_hdrt=rcm2rgrid(sat_hdrt@lat2d,sat_hdrt@lon2d,sat_hdrt,merge_lat,merge_lon,1) 
  regrid_hdrf=linint2(lon_0_hdrf(::skip),lat_0_hdrf(::-skip),sat_hdrf(::-skip,::skip),False,merge_lon+360,merge_lat,0)
  regrid_hdgf=linint2(lon_0_hdgf(::skip),lat_0_hdgf(::-skip),sat_hdgf(::-skip,::skip),False,merge_lon+360,merge_lat,0)

; handle missing values (if a grid point is missing in on data source, treat it as missing for all data sources)
  regrid_obs=where(regrid_hdrf.eq.1.e+20.or.regrid_hdrt.eq.-999.or.regrid_hdgf.eq.1.e+20, 99999., regrid_obs)
  regrid_hdrt=where(regrid_hdrf.eq.1.e+20.or.regrid_hdgf.eq.1.e+20.or.regrid_obs.eq.99999, 1.e+20, regrid_hdrt)
  regrid_hdgf=where(regrid_hdrf.eq.1.e+20.or.regrid_hdrt.eq.-999.or.regrid_obs.eq.99999, 1.e+20, regrid_hdgf)
  regrid_hdrf=where(regrid_hdrt.eq.-999.or.regrid_hdgf.eq.1.e+20.or.regrid_obs.eq.99999, 1.e+20, regrid_hdrf)
; name of the output NetCDF file
  output_file_name=case_info+"OBS_RT_RF_GF.large.nc"
  system("/bin/rm -f "+output_file_name)   ; remove any pre-existing file
  ncdf = addfile(output_file_name ,"c")  ; open output netCDF file
  regrid_obs!0="merge_lat"
  regrid_obs!1="merge_lon"
  regrid_obs&merge_lat=merge_lat
  regrid_obs&merge_lon=merge_lon

 copy_VarCoords(regrid_obs,regrid_hdgf)
 copy_VarCoords(regrid_obs,regrid_hdrf)
 copy_VarCoords(regrid_obs,regrid_hdrt)

ncdf->OBS=regrid_obs
ncdf->RT=regrid_hdrt
ncdf->RF=regrid_hdrf
ncdf->GF=regrid_hdgf

