load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
   
;purpose: given a netcdf of model and observed satellite images in netcdf format, 
;calculate the probability density function (PDF) and fraction-skill-scores (FSS) and write results into a netcdf file.
;2015 Shaowu Bao (sbao@coastal.edu)


;name of the netcdf file that contains the model and observed satellite data
f_name="satellite_obs_and_model_demo.nc"

;open image files
f=addfile(f_name,"r")

;load data: OBS is the observed; HDRT, HDRF and HDGF are three model generated synthetic satellite output. 
OBS=f->OBS
HDRT=f->RT
HDRF=f->RF
HDGF=f->GF

hdrt=HDRT(:,:)
hdrf=HDRF(:,:)
hdgf=HDGF(:,:)
obs=OBS(:,:)

;load the lat/lon coordinate
merge_lat=f->merge_lat
merge_lon=f->merge_lon


;calculate fss

;fss calculation from 230 to 310 K, threshold interval is 20K
;therefore the fss scores are for the ranges 0-230, 230-250, 250-270, 270-290 and 290-310.
;the fss window length varies from 0 to 6 in terms of grid points

min_threshold=230
max_threshold=310
d_threshold=20
max_Nthreshold=(max_threshold-min_threshold)/d_threshold+1
min_windows=0
max_windows=6
d_windows=1
max_Nwindows=(max_windows-min_windows)/d_windows+1


fss_data=new((/3,max_Nwindows,max_Nthreshold/),float)

print(max_Nwindows+" "+max_Nthreshold)
Nthreshold=0
do while (Nthreshold .lt. max_Nthreshold)

threshold=min_threshold+Nthreshold*d_threshold

 Nwindows=0
 do while (Nwindows .lt. max_Nwindows)
  
 windows=Nwindows*d_windows

 print("windows and threshold "+Nwindows+" "+Nthreshold)

 fss_hdrt=new((/dimsizes(merge_lat)-2*windows,dimsizes(merge_lon)-2*windows/),float)
 fss_hdrf=new((/dimsizes(merge_lat)-2*windows,dimsizes(merge_lon)-2*windows/),float)
 fss_hdgf=new((/dimsizes(merge_lat)-2*windows,dimsizes(merge_lon)-2*windows/),float)
 fss_obs =new((/dimsizes(merge_lat)-2*windows,dimsizes(merge_lon)-2*windows/),float)

 fss_hdrt=0.0
 fss_hdrf=0.0
 fss_hdgf=0.0
 fss_obs=0.0
 
 unit_hdrt=where(hdrt.le.threshold.and.hdrt.ge.threshold-d_threshold,1,0)
 unit_obs=where(obs.le.threshold.and.obs.ge.threshold-d_threshold,1,0)
 unit_hdrf=where(hdrf.le.threshold.and.hdrf.ge.threshold-d_threshold, 1, 0)
 unit_hdgf=where(hdgf.le.threshold.and.hdgf.ge.threshold-d_threshold, 1, 0)
     
 i=0
 do while(i.le.dimsizes(merge_lat)-2*windows-1)
 j=0
   do while(j.le.dimsizes(merge_lon)-2*windows-1)
     fss_hdrt(i,j)=sum(unit_hdrt(i:i+2*windows,j:j+2*windows))/(1+2*windows)^2
     fss_obs(i,j)=sum(unit_obs(i:i+2*windows,j:j+2*windows))/(1+2*windows)^2
     fss_hdrf(i,j)=sum(unit_hdrf(i:i+2*windows,j:j+2*windows))/(1+2*windows)^2
     fss_hdgf(i,j)=sum(unit_hdgf(i:i+2*windows,j:j+2*windows))/(1+2*windows)^2 
     j=j+1
   end do
   i=i+1
 end do

 FSS_hdrt=1-(sum((fss_obs-fss_hdrt)^2))/((sum(fss_obs^2)+sum(fss_hdrt^2)))
 FSS_hdrf=1-(sum((fss_obs-fss_hdrf)^2))/((sum(fss_obs^2)+sum(fss_hdrf^2)))
 FSS_hdgf=1-(sum((fss_obs-fss_hdgf)^2))/((sum(fss_obs^2)+sum(fss_hdgf^2)))

   fss_data(0,Nwindows,Nthreshold)=FSS_hdrt  
   fss_data(1,Nwindows,Nthreshold)=FSS_hdrf
   fss_data(2,Nwindows,Nthreshold)=FSS_hdgf

 delete([/fss_obs,fss_hdrt,fss_hdrf,fss_hdgf/])
 Nwindows=Nwindows+1
end do
Nthreshold=Nthreshold+1
end do 

; save data to netcdf files

; set the name of the output file that contains the output fss score values.
 output_file_name=f_name+"_thresholdrange_fss_pdf.nc"
 system("/bin/rm -f "+output_file_name)
; open output netCDF file
 ncdf = addfile(output_file_name ,"c")  
 filedimdef(ncdf,"time",-1,True)
;define the fss score data
  fss_data!1="scale"
  fss_data&scale=fspan(min_windows,max_windows,Nwindows)
  fss_data!2="threshhold"
  fss_data&threshhold=fspan(min_threshold, max_threshold, Nthreshold)
;write the fss score  data to the netcdf file
  ncdf->fss_data = fss_data


;calculat PDF
   option=True
;PDF calculation range: temperature 180K to 310K, divided into 50 bins
   option@bin_min=180
   option@bin_max=310
   obs_pdf =  pdfx(obs, 50, option)   ; default is 25
   hdrt_pdf = pdfx(hdrt,50, option)   ; default is 25
   hdrf_pdf = pdfx(hdrf,50, option)   ; default is 25
   hdgf_pdf = pdfx(hdgf,50, option)   ; default is 25
 obs_pdf!0="bin"
 obs_pdf&bin=fspan(1,50,50)
 copy_VarCoords(obs_pdf,hdrt_pdf)
 copy_VarCoords(obs_pdf,hdrf_pdf)
 copy_VarCoords(obs_pdf,hdgf_pdf)
;save the PDF values to the same output file where the fss scores are saved.
 ncdf->obs_pdf=obs_pdf
 ncdf->hdrt_pdf=hdrt_pdf
 ncdf->hdrf_pdf=hdrf_pdf
 ncdf->hdgf_pdf=hdgf_pdf

exit
